'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var rxjs = require('rxjs');
var operators = require('rxjs/operators');

var defaultOptions = {
  maxRetry: 3,
  delayMs: 500
};

var get = function get(obj, field, defaultValue) {
  return (obj || {})[field] || defaultValue;
};
/**
 *
 * Creat a fetch which retries when:
 * + GET with 5XX error
 * + Network error
 * @requires `rxjs`
 * @example
 * import {onErrorRetry} from 'higher-order-fetch/hofs/onErrorRetry'
 *
 * const fetch = onErrorRetry({
 *  maxRetry: 5,
 *  delayMs: 3000
 * })(window.fetch)
 *
 * fetch('http://example.com')// if request error, will retry maximum 5 times before resolve
 * .then(response => {
 *  // `response` is the last response after all retries or success.
 * })
 * @param {{maxRetry: number, delayMs: number}} [options]
 * @returns {function} a fetch-like function
 */


var onErrorRetry = function onErrorRetry(options) {
  return function (fetch) {
    return function (input) {
      var init = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
      var maxRetry = get(options, "maxRetry", defaultOptions.maxRetry);
      var delayMs = get(options, "delayMs", defaultOptions.delayMs);

      if (!init.method || init.method.toUpperCase() === "GET") {
        var count = 0;
        return rxjs.defer(function () {
          return rxjs.from(fetch(input, init).then(function (resp) {
            if ((resp.status + "").startsWith("5")) throw resp;
            return resp;
          }));
        }).pipe(operators.retryWhen(function (errors) {
          return errors.pipe(operators.mergeMap(function (error) {
            if (++count >= maxRetry) {
              return rxjs.throwError(error);
            } else {
              return rxjs.of(error).pipe(operators.delay(delayMs));
            }
          }));
        })).toPromise().then(function (resp) {
          return resp;
        }, function (resp) {
          if (resp.status === 500) return resp;
          throw resp;
        });
      }

      return fetch(input, init);
    };
  };
};

exports.onErrorRetry = onErrorRetry;
